@@ -27,15 +27,16 @@
 #include <QDBusInterface>
 #include <QDBusReply>
 #include <QUrl>
-#include <QDebug>
 #include <QHash>
 #include <QLocale>
 #include <QFileInfo>
 
+#include "kiocoredebug.h"
+
 enum BinaryUnitDialect {
     DefaultBinaryDialect = -1, ///< Used if no specific preference
-    IECBinaryDialect,          ///< KDE Default, KiB, MiB, etc. 2^(10*n)
-    JEDECBinaryDialect,        ///< KDE 3.5 default, KB, MB, etc. 2^(10*n)
+    IECBinaryDialect,          ///< KiB, MiB, etc. 2^(10*n)
+    JEDECBinaryDialect,        ///< KB, MB, etc. 2^(10*n)
     MetricBinaryDialect,       ///< SI Units, kB, MB, etc. 10^(3*n)
     LastBinaryDialect = MetricBinaryDialect
 };
@@ -49,11 +50,7 @@ BinaryUnitDialect _k_loadBinaryDialect()
 {
     KConfigGroup mainGroup(KSharedConfig::openConfig(), "Locale");
 
-    KConfig entryFile(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("locale/") + QString::fromLatin1("l10n/%1/entry.desktop").arg(QLocale::countryToString(QLocale().country()))));
-    entryFile.setLocale(QLocale::languageToString(QLocale().language()));
-    KConfigGroup entryGroup(&entryFile, "KCM Locale");
-
-    BinaryUnitDialect dialect = (BinaryUnitDialect) entryGroup.readEntry("BinaryUnitDialect", int(IECBinaryDialect));
+    BinaryUnitDialect dialect(BinaryUnitDialect(mainGroup.readEntry("BinaryUnitDialect", int(DefaultBinaryDialect))));
     dialect = (BinaryUnitDialect) mainGroup.readEntry("BinaryUnitDialect", int(dialect));
 
     // Error checking
@@ -67,69 +64,50 @@ BinaryUnitDialect _k_loadBinaryDialect()
 QStringList _k_loadBinaryDialectUnits()
 {
     BinaryUnitDialect dialect = *_k_defaultBinaryDialect();
+    // hack: i18nc expects all %-arguments to be replaced,
+    // so just pass a argument placeholder again
+    // That way just .arg(size) has to be called on the chosen unit string.
+    const QString dummyArgument = QStringLiteral("%1");
 
     // Choose appropriate units.
-    QList<QString> dialectUnits;
+    QStringList dialectUnits;
+    dialectUnits.reserve(9);
 
-    dialectUnits << i18nc("size in bytes", "%1 B");
+    dialectUnits << i18nc("size in bytes", "%1 B", dummyArgument);
 
     switch (dialect) {
     case MetricBinaryDialect:
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 1000 bytes", "%1 kB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 10^6 bytes", "%1 MB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 10^9 bytes", "%1 GB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 10^12 bytes", "%1 TB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 10^15 bytes", "%1 PB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 10^18 bytes", "%1 EB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 10^21 bytes", "%1 ZB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 10^24 bytes", "%1 YB");
+        dialectUnits << i18nc("size in 1000 bytes",  "%1 kB", dummyArgument)
+                     << i18nc("size in 10^6 bytes",  "%1 MB", dummyArgument)
+                     << i18nc("size in 10^9 bytes",  "%1 GB", dummyArgument)
+                     << i18nc("size in 10^12 bytes", "%1 TB", dummyArgument)
+                     << i18nc("size in 10^15 bytes", "%1 PB", dummyArgument)
+                     << i18nc("size in 10^18 bytes", "%1 EB", dummyArgument)
+                     << i18nc("size in 10^21 bytes", "%1 ZB", dummyArgument)
+                     << i18nc("size in 10^24 bytes", "%1 YB", dummyArgument);
         break;
 
     case JEDECBinaryDialect:
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 1024 bytes", "%1 KB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 2^20 bytes", "%1 MB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 2^30 bytes", "%1 GB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 2^40 bytes", "%1 TB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 2^50 bytes", "%1 PB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 2^60 bytes", "%1 EB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 2^70 bytes", "%1 ZB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("memory size in 2^80 bytes", "%1 YB");
+        dialectUnits << i18nc("memory size in 1024 bytes", "%1 KB", dummyArgument)
+                     << i18nc("memory size in 2^20 bytes", "%1 MB", dummyArgument)
+                     << i18nc("memory size in 2^30 bytes", "%1 GB", dummyArgument)
+                     << i18nc("memory size in 2^40 bytes", "%1 TB", dummyArgument)
+                     << i18nc("memory size in 2^50 bytes", "%1 PB", dummyArgument)
+                     << i18nc("memory size in 2^60 bytes", "%1 EB", dummyArgument)
+                     << i18nc("memory size in 2^70 bytes", "%1 ZB", dummyArgument)
+                     << i18nc("memory size in 2^80 bytes", "%1 YB", dummyArgument);
         break;
 
     case IECBinaryDialect:
     default:
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 1024 bytes", "%1 KiB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 2^20 bytes", "%1 MiB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 2^30 bytes", "%1 GiB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 2^40 bytes", "%1 TiB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 2^50 bytes", "%1 PiB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 2^60 bytes", "%1 EiB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 2^70 bytes", "%1 ZiB");
-        // i18n: Dumb message, avoid any markup or scripting.
-        dialectUnits << i18nc("size in 2^80 bytes", "%1 YiB");
+        dialectUnits << i18nc("size in 1024 bytes", "%1 KiB", dummyArgument)
+                     << i18nc("size in 2^20 bytes", "%1 MiB", dummyArgument)
+                     << i18nc("size in 2^30 bytes", "%1 GiB", dummyArgument)
+                     << i18nc("size in 2^40 bytes", "%1 TiB", dummyArgument)
+                     << i18nc("size in 2^50 bytes", "%1 PiB", dummyArgument)
+                     << i18nc("size in 2^60 bytes", "%1 EiB", dummyArgument)
+                     << i18nc("size in 2^70 bytes", "%1 ZiB", dummyArgument)
+                     << i18nc("size in 2^80 bytes", "%1 YiB", dummyArgument);
         break;
     }
 
@@ -139,7 +117,7 @@ QStringList _k_loadBinaryDialectUnits()
 KIOCORE_EXPORT QString KIO::convertSize(KIO::filesize_t fileSize)
 {
     const BinaryUnitDialect dialect = *_k_defaultBinaryDialect();
-    const QStringList dialectUnits = *_k_defaultBinaryDialectUnits();
+    const QStringList &dialectUnits = *_k_defaultBinaryDialectUnits();
     double size = fileSize;
     int unit = 0; // Selects what unit to use from cached list
     double multiplier = 1024.0;
@@ -153,12 +131,8 @@ KIOCORE_EXPORT QString KIO::convertSize(KIO::filesize_t fileSize)
         unit++;
     }
 
-    if (unit == 0) {
-        // Bytes, no rounding
-        return dialectUnits[unit].arg(QLocale().toString(size, 'f', 0));
-    } else {
-        return dialectUnits[unit].arg(QLocale().toString(size, 'f', 1));
-    }
+    const int precision = (unit == 0) ? 0 : 1; // unit == 0 -> Bytes, no rounding
+    return dialectUnits.at(unit).arg(QLocale().toString(size, 'f', precision));
 }
 
 KIOCORE_EXPORT QString KIO::convertSizeFromKiB(KIO::filesize_t kibSize)
@@ -199,7 +173,7 @@ KIOCORE_EXPORT QString KIO::convertSeconds(unsigned int seconds)
     }
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOCORE_NO_DEPRECATED
 KIOCORE_EXPORT QTime KIO::calculateRemaining(KIO::filesize_t totalSize, KIO::filesize_t processedSize, KIO::filesize_t speed)
 {
     QTime remainingTime;
@@ -291,7 +265,7 @@ KIO::CacheControl KIO::parseCacheControl(const QString &cacheControl)
         return KIO::CC_Reload;
     }
 
-    qDebug() << "unrecognized Cache control option:" << cacheControl;
+    qCDebug(KIO_CORE) << "unrecognized Cache control option:" << cacheControl;
     return KIO::CC_Verify;
 }
 
@@ -312,7 +286,7 @@ QString KIO::getCacheControlString(KIO::CacheControl cacheControl)
     if (cacheControl == KIO::CC_Reload) {
         return "Reload";
     }
-    qDebug() << "unrecognized Cache control enum value:" << cacheControl;
+    qCDebug(KIO_CORE) << "unrecognized Cache control enum value:" << cacheControl;
     return QString();
 }
 
@@ -413,43 +387,38 @@ QUrl KIO::upUrl(const QUrl &url)
 
 QString KIO::suggestName(const QUrl &baseURL, const QString &oldName)
 {
-    QString dotSuffix, suggestedName;
-    QString basename = oldName;
-    const QChar spacer(' ');
+    QString basename;
 
-    //ignore dots at the beginning, that way "..aFile.tar.gz" will become "..aFile 1.tar.gz" instead of " 1..aFile.tar.gz"
-    int index = basename.indexOf('.');
-    int continous = 0;
-    while (continous == index) {
-        index = basename.indexOf('.', index + 1);
-        ++continous;
+    // Extract the original file extension from the filename
+    QMimeDatabase db;
+    QString nameSuffix = db.suffixForFileName(oldName);
+
+    if (oldName.lastIndexOf('.') == 0) {
+        basename = ".";
+        nameSuffix = oldName;
+    } else if (nameSuffix.isEmpty()) {
+        basename = oldName;
+    } else {
+        nameSuffix.prepend('.');
+        basename = oldName.left(oldName.length() - nameSuffix.length());
     }
 
-    if (index != -1) {
-        dotSuffix = basename.mid(index);
-        basename.truncate(index);
-    }
-
-    int pos = basename.lastIndexOf(spacer);
-
-    if (pos != -1) {
-        QString tmp = basename.mid(pos + 1);
-        bool ok;
-        int number = tmp.toInt(&ok);
-
-        if (!ok) {  // ok there is no number
-            suggestedName = basename + spacer + '1' + dotSuffix;
-        } else {
-            // yes there's already a number behind the spacer so increment it by one
-            basename.replace(pos + 1, tmp.length(), QString::number(number + 1));
-            suggestedName = basename + dotSuffix;
-        }
-    } else { // no spacer yet
-        suggestedName = basename + spacer + "1" + dotSuffix;
+    // check if (number) exists from the end of the oldName and increment that number
+    QRegExp numSearch("\\(\\d+\\)");
+    int start = numSearch.lastIndexIn(oldName);
+    if (start != -1) {
+        QString numAsStr = numSearch.cap(0);
+        QString number = QString::number(numAsStr.mid(1, numAsStr.size() - 2).toInt() + 1);
+        basename = basename.left(start) + '(' + number + ')';
+    } else {
+        // number does not exist, so just append " (1)" to filename
+        basename += " (1)";
     }
+    const QString suggestedName = basename + nameSuffix;
 
     // Check if suggested name already exists
     bool exists = false;
+
     // TODO: network transparency. However, using NetAccess from a modal dialog
     // could be a problem, no? (given that it uses a modal widget itself....)
     if (baseURL.isLocalFile()) {
